<?php

namespace Litecms\Game\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Game\Http\Requests\GamesRequest;
use Litecms\Game\Interfaces\GamesRepositoryInterface;
use Litecms\Game\Models\Games;

/**
 * Resource controller class for games.
 */
class GamesResourceController extends BaseController
{

    /**
     * Initialize games resource controller.
     *
     * @param type GamesRepositoryInterface $games
     *
     * @return null
     */
    public function __construct(GamesRepositoryInterface $games)
    {
        parent::__construct();
        $this->repository = $games;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Game\Repositories\Criteria\GamesResourceCriteria::class);
    }

    /**
     * Display a list of games.
     *
     * @return Response
     */
    public function index(GamesRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Game\Repositories\Presenter\GamesPresenter::class)
                ->$function();
        }

        $games = $this->repository->paginate();

        return $this->response->title(trans('game::games.names'))
            ->view('game::games.index', true)
            ->data(compact('games'))
            ->output();
    }

    /**
     * Display games.
     *
     * @param Request $request
     * @param Model   $games
     *
     * @return Response
     */
    public function show(GamesRequest $request, Games $games)
    {

        if ($games->exists) {
            $view = 'game::games.show';
        } else {
            $view = 'game::games.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('game::games.name'))
            ->data(compact('games'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new games.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(GamesRequest $request)
    {

        $games = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('game::games.name')) 
            ->view('game::games.create', true) 
            ->data(compact('games'))
            ->output();
    }

    /**
     * Create new games.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(GamesRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $games                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('game::games.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('game/games/' . $games->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/game/games'))
                ->redirect();
        }

    }

    /**
     * Show games for editing.
     *
     * @param Request $request
     * @param Model   $games
     *
     * @return Response
     */
    public function edit(GamesRequest $request, Games $games)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('game::games.name'))
            ->view('game::games.edit', true)
            ->data(compact('games'))
            ->output();
    }

    /**
     * Update the games.
     *
     * @param Request $request
     * @param Model   $games
     *
     * @return Response
     */
    public function update(GamesRequest $request, Games $games)
    {
        try {
            $attributes = $request->all();

            $games->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('game::games.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('game/games/' . $games->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('game/games/' . $games->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the games.
     *
     * @param Model   $games
     *
     * @return Response
     */
    public function destroy(GamesRequest $request, Games $games)
    {
        try {

            $games->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('game::games.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('game/games/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('game/games/' . $games->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple games.
     *
     * @param Model   $games
     *
     * @return Response
     */
    public function delete(GamesRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('game::games.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('game/games'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/game/games'))
                ->redirect();
        }

    }

    /**
     * Restore deleted games.
     *
     * @param Model   $games
     *
     * @return Response
     */
    public function restore(GamesRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('game::games.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/game/games'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/game/games/'))
                ->redirect();
        }

    }

}
